//
// C++ Implementation: njbmediadevice
//
// Description: This class is used to manipulate Nomad Creative Jukebox and others media player that works with the njb libraries.
//
//
// Author: Andres Oton <andres.oton@gmail.com>, (C) 2006
//
// Modified by: T.R.Shashwath <trshash84@gmail.com>
//
// Based on kionjb (http://sourceforge.net/projects/kionjb) and kzenexplorer (http://sourceforge.net/projects/kzenexplorer)
// Copyright: See COPYING file that comes with this distribution
//

#include "njbmediadevice.h"

AMAROK_EXPORT_PLUGIN( NjbMediaDevice )

#include "debug.h"
#include "metabundle.h"
#include "collectiondb.h"
#include "statusbar/statusbar.h"
#include "statusbar/popupMessage.h"

#include <quuid.h>

// kde
#include <kapplication.h>
#include <kdebug.h>
#include <kinstance.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <ktempfile.h>
#include <kurl.h>
#include <kiconloader.h>       //smallIcon
#include <kpopupmenu.h>
#include <kfiledialog.h>
#include <qlistview.h>
#include <qdir.h>

// posix
#include <stdint.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>

#include <kurlrequester.h>     //downloadSelectedItems()
#include <kurlrequesterdlg.h>  //downloadSelectedItems()

namespace amaroK { extern KConfig *config( const QString& ); }

// This function has NOT handled the request, so other functions may be called
// upon to do so
const int NJB_NOTHANDLED = 0;

// This function has handled the request, so no further processing is needed.
const int NJB_HANDLED = -1;

njb_t* theNjb = NULL;
trackValueList* theTracks = NULL;

class NjbMediaItem : public MediaItem
{
    public:
        NjbMediaItem( QListView *parent, QListViewItem *after = 0 )
            : MediaItem( parent, after )
        {isdownload = false;}

        NjbMediaItem( QListViewItem *parent, QListViewItem *after = 0 )
            : MediaItem( parent, after )
        {isdownload=false;}

        void setId(unsigned id) {trid=id;}

        unsigned getId() {return trid;}

        QString getFileName() {return filename;}

        void setFileName(QString name) {filename = name;}

        void setIsDownloadItem(bool d) {isdownload = d;}

        bool IsDownloadItem() {return isdownload;}

    private:

        unsigned trid;
        QString filename;
        bool isdownload;
};


NjbMediaDevice::NjbMediaDevice(): MediaDevice()
{

    //	listAmarokPlayLists = new QListView();
    m_name = "NJB Media device";
    theNjb = m_njb = NULL;
    m_captured = false;
    m_libcount = 0;
    theTracks = &trackList;
    m_connected = false;
    m_td = 0;
    NJB_Set_Debug(0); // or try DD_SUBTRACE

}


NjbMediaDevice::~NjbMediaDevice()
{

}

bool NjbMediaDevice::closeDevice()
{
    DEBUG_BLOCK

    if(m_captured) {
        NJB_Release( m_njb);
        m_captured = false;
    }
    m_connected = false;

    if( m_njb ) {
        debug() << ": disconnecting. Is captured: "<< m_captured << endl;
        /*		if(m_captured)
                        unlockDevice();*/

        NJB_Close( m_njb);
        debug() << ": deleting m_njb " << m_njb << endl;
        delete m_njb;
        m_njb = NULL;
        theNjb = m_njb;
    }

    debug()<< ": disconnected, pid=" << getpid() << endl;

    clearItems();

    return true;
}

void NjbMediaDevice::unlockDevice()
{
    DEBUG_BLOCK
}

bool NjbMediaDevice::getCapacity(KIO::filesize_t* total, KIO::filesize_t* available)
{
    DEBUG_BLOCK

    if(!m_connected)
	return false;


    u_int64_t itotal;
    u_int64_t ifree;

    if(NJB_Get_Disk_Usage(m_njb, &itotal, &ifree) == -1)
	return false;

    *total = itotal;
    *available = ifree;

    return true;

}

//DONE
bool NjbMediaDevice::isConnected()
{
    return m_connected;

}

bool NjbMediaDevice::isPlayable(const MetaBundle& bundle)
{
    DEBUG_BLOCK
    debug() << ": pid=" << getpid() << endl;
    if(bundle.fileType() == MetaBundle::mp3)
        return true;

    return false;
}

bool NjbMediaDevice::isPreferredFormat(const MetaBundle& bundle)
{
    DEBUG_BLOCK
    debug() << ": pid=" << getpid() << endl;
    if(bundle.fileType() == MetaBundle::mp3)
        return true;
    else
        return false;
}

bool NjbMediaDevice::lockDevice(bool tryOnly)
{
    DEBUG_BLOCK
    debug() << ": pid=" << getpid() << endl;

    return true;
}

//DONE
bool NjbMediaDevice::openDevice(bool)
{
    DEBUG_BLOCK
    debug() << ": pid=" << getpid() << endl;

    if( m_njb )
        return true;

    QString genericError = i18n( "Could not connect to Nomad device" );

    int n;
    if( NJB_Discover( njbs, 0, &n) == -1 || n == 0) {
        amaroK::StatusBar::instance()->shortLongMessage( genericError, i18n("Nomad: Connecting "), KDE::StatusBar::Error );
        debug() << ": no NJBs found\n";
        theNjb = m_njb = NULL;
        return false;
    }

    m_njb = new njb_t;
    *m_njb = njbs[0];
    theNjb = m_njb;

    if( NJB_Open( m_njb) == -1) {
        debug() << ": couldn't open\n";
        debug() << ": deleting " << m_njb << "\n";
        delete m_njb;
        theNjb = m_njb = NULL;
        return false;
    }
    m_connected = true;

    debug() << ": m_njb " << m_njb << "\n";

    debug() << ": pid=" << getpid() << "Capturing." << endl;

    if( NJB_Capture(m_njb) == -1) {
        debug() << ": couldn't capture\n";
        m_captured = false;
    }
    else
        m_captured = true;

    if(m_captured)
    {
        readJukeboxMusic( );
	QString s;
	s.sprintf("%d tracks found", trackList.size());
        amaroK::StatusBar::instance()->shortLongMessage( s, s, KDE::StatusBar::Information );
    }

    return true;

}

int NjbMediaDevice::deleteFromDevice(unsigned id)
{
    debug() << ": pid=" << getpid() << endl;

    if(lockDevice( false) )
        return -1;

    int status = NJB_Delete_Track( m_njb, id );

    if( status != NJB_SUCCESS) {
        debug() << ": NJB_Delete_Track failed" << endl;
        return -1;
    }

    // remove from the cache
    trackList.remove(trackList.findTrackById( id ) );

    return 1;
}

int NjbMediaDevice::deleteItemFromDevice(MediaItem* item, bool onlyPlayed)
{
    Q_UNUSED(onlyPlayed)
    //onlyPlayed is ignored because this device doesn't suppor directories
    //and not store the number of times that a file is played

    debug() << endl;

    NjbMediaItem *njbItem = dynamic_cast<NjbMediaItem *>(item);
    if(item->type() == MediaItem::ARTIST)
        return deleteArtist(njbItem);

    if(item->type() == MediaItem::ALBUM)
        return deleteAlbum(njbItem);

    if(item->type() == MediaItem::TRACK)
        return deleteTrack( njbItem );

    debug() << ": OK" << endl;

    return -1;
}

int NjbMediaDevice::deleteArtist(NjbMediaItem *artistItem)
{
    debug() << endl;
    if(artistItem->IsDownloadItem())
    {
        delete artistItem;
        return 1;
    }

    int itemsDeleted = 0;
    if(artistItem->type() == MediaItem::ARTIST)
    {
        NjbMediaItem* auxItem = dynamic_cast<NjbMediaItem *>(artistItem->firstChild());
        while(auxItem)
        {
            itemsDeleted += deleteAlbum( auxItem );
            auxItem = dynamic_cast<NjbMediaItem *>(auxItem->nextSibling());
        }
    }
    delete artistItem;
    return itemsDeleted;
}

int NjbMediaDevice::deleteAlbum(NjbMediaItem *albumItem)
{
    debug() << endl;

    if(albumItem->IsDownloadItem())
    {
        delete albumItem;
        return 1;
    }
    int itemsDeleted = 0;
    if(albumItem->type() == MediaItem::ALBUM)
    {
        NjbMediaItem *auxItem = dynamic_cast<NjbMediaItem *>(albumItem->firstChild());
        while(auxItem)
        {
            itemsDeleted += deleteTrack(auxItem);
            auxItem = dynamic_cast<NjbMediaItem *>(auxItem->nextSibling());
        }
    }
    delete albumItem;
    return itemsDeleted;
}

int NjbMediaDevice::deleteTrack(NjbMediaItem *trackItem)
{
    debug() << endl;

    if(trackItem->IsDownloadItem())
    {
        delete trackItem;
        return 1;
    }

    if(!lockDevice( false ) )
        return -1;

    int status = NJB_Delete_Track( m_njb, trackItem->getId());

    if( status != NJB_SUCCESS) {
        debug() << ": NJB_Delete_Track failed" << endl;
        return -1;
    }

    debug() << ": NJB_Delete_Track track deleted" << endl;

    // remove from the cache
    trackList.remove(trackList.findTrackById( trackItem->getId() ) );

    unlockDevice();

    //	readJukeboxMusic();
    delete trackItem;
    return 1;
}

int NjbMediaDevice::downloadSelectedItems( NjbMediaItem * item )
{
    debug() << endl;

    /* Copied from ifpmediadevice */
    QString save = QString::null;

    KURLRequesterDlg dialog( save, 0, 0 );
    dialog.setCaption( kapp->makeStdCaption( i18n( "Choose a Download Directory" ) ) );
    dialog.urlRequester()->setMode( KFile::Directory | KFile::ExistingOnly );
    dialog.exec();

    KURL destDir = dialog.selectedURL();
    if( destDir.isEmpty() )
	    return -1;

    destDir.adjustPath( 1 ); //add trailing slash

    NjbMediaItem *njbItem = dynamic_cast<NjbMediaItem *>(item);
    if(item->type() == MediaItem::ARTIST)
        return downloadArtist(njbItem, destDir);

    if(item->type() == MediaItem::ALBUM)
        return downloadAlbum(njbItem, destDir);

    if(item->type() == MediaItem::TRACK)
        return downloadTrack( njbItem, destDir );

    debug() << ": OK" << endl;

    return -1;

}

int NjbMediaDevice::downloadArtist(NjbMediaItem *artistItem, KURL destDir)
{
    debug() << "downloadArtist" <<  endl;

    int itemsDownload = 0;

    if(artistItem->type() == MediaItem::ARTIST)
    {
        NjbMediaItem* auxItem = dynamic_cast<NjbMediaItem *>(artistItem->firstChild());
        while(auxItem)
        {
            itemsDownload += downloadAlbum( auxItem, destDir );
            auxItem = dynamic_cast<NjbMediaItem *>(auxItem->nextSibling());
        }
    }

    return itemsDownload;
}

int NjbMediaDevice::downloadAlbum(NjbMediaItem *albumItem, KURL destDir)
{
    debug() << "downloadAlbum:" << endl;

    int itemsDownload = 0;

    if(albumItem->type() == MediaItem::ALBUM)
    {
        NjbMediaItem* auxItem = dynamic_cast<NjbMediaItem *>(albumItem->firstChild());

        while(auxItem)
        {
            itemsDownload += downloadTrack( auxItem, destDir );
            auxItem = dynamic_cast<NjbMediaItem *>(auxItem->nextSibling());
        }
    }
    return itemsDownload;
}

int NjbMediaDevice::downloadTrack(NjbMediaItem *trackItem, KURL destDir)
{
    debug() << "downloadTrack:" << endl;

    if(trackItem->type() == MediaItem::TRACK)
    {
        debug() << "Artist: " << trackItem->parent()->parent()->text(0) << "  Album: " << trackItem->parent()->text(0) << endl;


        downloadTrackNow( trackItem, destDir.path() );

        return 1;
    }
    return 0;
}


int NjbMediaDevice::downloadTrackNow( NjbMediaItem *item , QString path)
{
    debug() << endl;

    if(!lockDevice(false))
        return 0;

    //int   NJB_Get_Track (njb_t *njb, u_int32_t trackid, u_int32_t size, const char *path, NJB_Xfer_Callback *callback, void *data)
    path += item->bundle()->artist();
    debug() << "Getting track: "<< path << endl;
    QDir dir;
    if( !dir.exists( path ) )
    {
        dir.mkdir(path);
    }
    path = path + "/" + item->bundle()->album();
    if( !dir.exists( path ) )
    {
        dir.mkdir(path);
    }
    path = path + "/" + item->getFileName();
    if(NJB_Get_Track (m_njb, item->getId(), item->bundle()->filesize(), path.latin1(), progressCallback, this) != NJB_SUCCESS)
    {
        debug() << ": NJB_Send_Track failed\n";
        if (NJB_Error_Pending(m_njb))
        {
            const char* njbError;
            while ((njbError = NJB_Error_Geterror(m_njb)))
                warning() << ": " << njbError << endl;
        }
        else
            debug() << ": No reason for failure reported.\n";

        m_busy = false;

        return 0;
    }

    return 1;
}

MediaItem* NjbMediaDevice::copyTrackToDevice(const MetaBundle& bundle)
{
    //TODO:REVIEW
    debug() << ": pid=" << getpid() << endl;

    if(!lockDevice( false) )
        return 0;

    trackValueList::const_iterator it_track = theTracks->findTrackByName( bundle.filename() );
    if( it_track != theTracks->end() ) {
        deleteFromDevice( (*it_track).getId() );
    }

    if(!lockDevice( false) )
        return 0;

    // read the mp3 header
    int duration = bundle.length();
    debug() << ": URL: " << bundle.prettyURL() << endl;

    if( !duration ) {
        m_errMsg = i18n( "Not a valid mp3 file");
        return 0;
    }

    NjbTrack *taggedTrack = new NjbTrack();
    taggedTrack->setSize( bundle.filesize() );
    taggedTrack->setDuration( bundle.length() );
    taggedTrack->setFilename( bundle.filename() );
    taggedTrack->setTitle( bundle.title() );
    taggedTrack->setGenre( bundle.genre() );
    taggedTrack->setArtist( bundle.artist() );
    taggedTrack->setAlbum( bundle.album() );
    taggedTrack->setCodec( "mp3" );
    taggedTrack->setTrackNum( bundle.track() );
    taggedTrack->setYear( QString::number( bundle.year() ) );

    // send the track
    // totalSize( taggedTrack.getSize() );
    debug() << "copyTrack: sending..." << endl;
    debug() << "copyTrack: "
        << taggedTrack->getTitle() << " " << taggedTrack->getAlbum() << " "
        << taggedTrack->getGenre() << " "
        << "size:" << taggedTrack->getSize() << " "
        << taggedTrack->getArtist() << endl;

    u_int32_t id;
    m_progressStart = time( 0);
    m_progressMessage = "Copying / Sent %1%...";

    njb_songid_t* songid = NJB_Songid_New();
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Filename( bundle.filename().latin1() ) );
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Filesize( taggedTrack->getSize() ));
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Codec( taggedTrack->getCodec().latin1() ));
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Title( taggedTrack->getTitle().latin1() ));
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Album(taggedTrack->getAlbum().latin1()));
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Genre(taggedTrack->getGenre().latin1()));
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Artist(taggedTrack->getArtist().latin1()));
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Length(taggedTrack->getDuration()));
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Tracknum(taggedTrack->getTrackNum()));
    NJB_Songid_Addframe(songid, NJB_Songid_Frame_New_Year(taggedTrack->getYear().toUInt()));

    m_busy = true;
    debug() << ": m_njb is " << m_njb << "\n";
    kapp->processEvents( 100 );
    if(NJB_Send_Track (m_njb, bundle.url().path().latin1(), songid, progressCallback, this, &id) != NJB_SUCCESS)
    {
        debug() << ": NJB_Send_Track failed\n";
        if (NJB_Error_Pending(m_njb))
        {
            const char* njbError;
            while ((njbError = NJB_Error_Geterror(m_njb)))
                warning() << ": " << njbError << endl;
        }
        else
            debug() << ": No reason for failure reported.\n";

        m_busy = false;
        NJB_Songid_Destroy( songid );

        return 0;
    }

    m_busy = false;
    NJB_Songid_Destroy( songid );

    taggedTrack->setId( id );

    // cache the track
    trackList.append( *taggedTrack );;

    //TODO: Construct a MediaItem

    return addTrackToView( taggedTrack );

}

MediaItem* NjbMediaDevice::newPlaylist(const QString& name, MediaItem* parent, QPtrList< MediaItem > items)
{
    DEBUG_BLOCK
    debug() << ": pid=" << getpid() << endl;

    MediaItem* newplaylist = new MediaItem(parent);

    NjbPlaylist playlist;
    int status = playlist.setName( name );

    if(status == NJB_SUCCESS)
    {
        NjbMediaItem *newNjbPlayList = new NjbMediaItem(listAmarokPlayLists);
        for(MediaItem *item=items.first();item;item=items.next())
        {

            status = playlist.addTrack( item->bundle()->filename() );
            if( status == NJB_FAILURE)
            {
                //TODO: Show a message with the error.
            }
            else if( status != NJB_SUCCESS)
            {
                return 0;
            }


            NjbMediaItem *nitem = dynamic_cast<NjbMediaItem *>(item);

        }

        status = playlist.update();
        if( status != NJB_SUCCESS)
        {
            return 0;
        }

    }

    //TODO:Crear un conversor de Playlists a las listas de Amarok
    return 0;
}

QStringList NjbMediaDevice::supportedFiletypes()
{
    return MediaDevice::supportedFiletypes();
}

TransferDialog* NjbMediaDevice::getTransferDialog()
{
    return m_td;
}

void NjbMediaDevice::addConfigElements(QWidget* arg1)
{
    MediaDevice::addConfigElements(arg1);
}

void NjbMediaDevice::addToDirectory(MediaItem* directory, QPtrList< MediaItem > items)
{
    MediaDevice::addToDirectory(directory, items);
}

void NjbMediaDevice::addToPlaylist(MediaItem* playlist, MediaItem* after, QPtrList< MediaItem > items)
{
    MediaDevice::addToPlaylist(playlist, after, items);
}

void NjbMediaDevice::applyConfig()
{
    MediaDevice::applyConfig();
}

void NjbMediaDevice::cancelTransfer()
{
    MediaDevice::cancelTransfer();
}

// DONE
void NjbMediaDevice::init(MediaBrowser* parent)
{
    MediaDevice::init(parent);
}

void NjbMediaDevice::loadConfig()
{
    MediaDevice::loadConfig();
}

void NjbMediaDevice::removeConfigElements(QWidget* arg1)
{
    MediaDevice::removeConfigElements(arg1);
}

void NjbMediaDevice::rmbPressed(QListViewItem* qitem, const QPoint& point, int )
{

    enum Actions { DOWNLOAD, RENAME, DELETE};

    NjbMediaItem *item = static_cast<NjbMediaItem *>(qitem);
    if ( item )
    {
        KPopupMenu menu( m_view);
        menu.insertItem( SmallIconSet( amaroK::icon( "collection" ) ), i18n("Download"), DOWNLOAD );
        menu.insertSeparator();
        //			menu.insertItem( SmallIconSet( amaroK::icon( "edit" ) ), i18n( "Rename" ), RENAME );
        menu.insertItem( SmallIconSet( amaroK::icon( "remove" ) ), i18n( "Delete" ), DELETE );


        int id =  menu.exec( point );
        switch( id )
        {
        case DOWNLOAD:
            debug() << "Downloading" << endl;
            downloadSelectedItems( item );
            break;

        case RENAME:
            //TODO: Implement rename tracks
            //m_view->rename( item, 0 );
            break;

        case DELETE:
            deleteItemFromDevice( item , false);
            break;
        }
        return;
    }

}

void NjbMediaDevice::runTransferDialog()
{
    m_td = new TransferDialog( this );
    m_td->exec();
}

void NjbMediaDevice::synchronizeDevice()
{
}

void NjbMediaDevice::updateRootItems()
{
    MediaDevice::updateRootItems();
}

void NjbMediaDevice::hideProgress()
{
    MediaDevice::hideProgress();
}

void NjbMediaDevice::purgeEmptyItems(MediaItem* root)
{
    MediaDevice::purgeEmptyItems(root);
}

void NjbMediaDevice::setConfigBool(const QString& name, bool value)
{
    MediaDevice::setConfigBool(name, value);
}

void NjbMediaDevice::setConfigString(const QString& name, const QString& value)
{
    MediaDevice::setConfigString(name, value);
}

void NjbMediaDevice::setDeviceType(const QString& type)
{
    MediaDevice::setDeviceType(type);
}

void NjbMediaDevice::setFirstSort(QString text)
{
    MediaDevice::setFirstSort(text);
}

void NjbMediaDevice::setSecondSort(QString text)
{
    MediaDevice::setSecondSort(text);
}

void NjbMediaDevice::setSpacesToUnderscores(bool yesno)
{
    MediaDevice::setSpacesToUnderscores(yesno);
}

void NjbMediaDevice::setThirdSort(QString text)
{
    MediaDevice::setThirdSort(text);
}

void NjbMediaDevice::syncStatsFromDevice(MediaItem* root)
{
    MediaDevice::syncStatsFromDevice(root);
}

void NjbMediaDevice::syncStatsToDevice(MediaItem* root)
{
    MediaDevice::syncStatsToDevice(root);
}

int NjbMediaDevice::progressCallback(  u_int64_t sent, u_int64_t total, const char* /*buf*/, unsigned /*len*/, void* data)
{
    kapp->processEvents( 100 );

    NjbMediaDevice *njb_media = reinterpret_cast<NjbMediaDevice*>(data);

    if( njb_media->isCancelled() )
    {
        debug() << "Cancelling transfer operation" << endl;
        njb_media->setCancelled( false );
        njb_media->setProgress( sent, total );
        return 1;
    }

    njb_media->setProgress( sent, total);

    return 0;
}

void NjbMediaDevice::clearItems()
{
    NjbMediaItem *auxItem = dynamic_cast<NjbMediaItem *>( m_view->firstChild() );
    while(auxItem)
    {
        NjbMediaItem *nextItem = dynamic_cast<NjbMediaItem *>(auxItem->nextSibling());
        delete auxItem;
        auxItem = nextItem;
    }
}

/* ------------------------------------------------------------------------ */
/** Transfer musical info from the njb to local structures */
int NjbMediaDevice::readJukeboxMusic( void)
{
    int result = NJB_SUCCESS;

    // First, read jukebox tracks
    if(trackList.isEmpty()) {

        result = trackList.readFromDevice();
    }

    if(result == NJB_SUCCESS)
    {
        clearItems();

        /*m_playlistItem = new NjbMediaItem( m_view );
        m_playlistItem->setText( 0, i18n("Playlists") );
        m_playlistItem->m_order = -5;
        m_playlistItem->setType( MediaItem::PLAYLISTSROOT );*/

        kapp->processEvents( 100 );

        trackValueList::iterator it;
        for( it = trackList.begin(); it != trackList.end(); it++)
        {

            addTrackToView( &(*it) );

            kapp->processEvents( 100 );
        }

        unlockDevice();
    }


    debug() << ": return " << result << endl;
    return result;
}

NjbMediaItem *
NjbMediaDevice::addTrackToView(NjbTrack *track, NjbMediaItem *item)
{
    QString artistName;
    artistName = track->getArtist();

    NjbMediaItem *artist = getArtist(artistName);
    if(!artist)
    {
        artist = new NjbMediaItem(m_view);
        artist->setText( 0, artistName );
        artist->setType( MediaItem::ARTIST );
    }

    QString albumName = track->getAlbum();
    NjbMediaItem *album = (NjbMediaItem*)artist->findItem(albumName);
    if(!album)
    {
        album = new NjbMediaItem( artist );
        album->setText( 0, albumName );
        album->setType( MediaItem::ALBUM );
    }

    if( item )
        album->insertItem( item );
    else
    {
        item = new NjbMediaItem( album );

        QString titleName = track->getTitle();

        item->setText( 0, titleName );
        item->setType( MediaItem::TRACK );
        item->setBundle( track->getMetaBundle());
        item->setId( track->getId() );
        item->setFileName( track->getFilename() );
    }
    return item;

}

NjbMediaItem *
NjbMediaDevice::getArtist(const QString &artist)
{
    for(NjbMediaItem *it = dynamic_cast<NjbMediaItem *>(m_view->firstChild());
            it;
            it = dynamic_cast<NjbMediaItem *>(it->nextSibling()))
    {
        if(it->m_type==MediaItem::ARTIST && artist == it->text(0))
            return it;
    }

    return 0;
}

NjbMediaItem *
NjbMediaDevice::getAlbum(const QString &artist, const QString &album)
{
    NjbMediaItem *item = getArtist(artist);
    if(item)
        return dynamic_cast<NjbMediaItem *>(item->findItem(album));

    return 0;
}

